@@ -67,11 +67,11 @@ def sphinxify(docstring, format='html'):
 
         sage: from sagenb.misc.sphinxify import sphinxify
         sage: sphinxify('A test')
-        '\n<div class="docstring">\n    \n  <p>A test</p>\n\n\n</div>'
+        '...<div class="docstring">\n    \n  <p>A test</p>\n\n\n</div>'
         sage: sphinxify('**Testing**\n`monospace`')
-        '\n<div class="docstring"...<strong>Testing</strong>\n<span class="math"...</p>\n\n\n</div>'
+        '...<div class="docstring"...<strong>Testing</strong>\n<span class="math"...</p>\n\n\n</div>'
         sage: sphinxify('`x=y`')
-        '\n<div class="docstring">\n    \n  <p><span class="math">x=y</span></p>\n\n\n</div>'
+        '...<div class="docstring">\n    \n  <p><span class="math">x=y</span></p>\n\n\n</div>'
         sage: sphinxify('`x=y`', format='text')
         'x=y\n'
         sage: sphinxify(':math:`x=y`', format='text')
